<?php

/**
 * @file
 * Handle the base functionality for querying the Pin API.
 */


/**
 * A simple wrapper class for our endpoints.
 */
class CommercePinEndpoints {
  const ENDPOINT_LIVE = 'api.pin.net.au/1';
  const ENDPOINT_TEST = 'test-api.pin.net.au/1';
}


/**
 * The base class for a Pin transaction.
 */
abstract class CommercePinTransactionBase {

  protected $secret_key = FALSE;

  /**
   * @param $secret_key
   *   The secret key used to query the API.
   * @param bool $test_mode
   *   If we are going to query the API in test mode.
   */
  function __construct($secret_key, $test_mode = FALSE) {
    $this->secret_key = $secret_key;
    $this->test_mode = $test_mode;
  }

  /**
   * @return String
   *   The endpoint for the transaction including the authentication details.
   */
  function getApiBase() {

    $url = $this->test_mode
      ? CommercePinEndpoints::ENDPOINT_TEST
      : CommercePinEndpoints::ENDPOINT_LIVE;

    return 'https://' . $this->secret_key . ':@' . $url;
  }

  /**
   * Query the Pin API.
   *
   * @param $endpoint
   *   The endpoint we are querying.
   * @param $variables
   *   A list of variables to send to the gateway.
   * @return CommercePinResponse
   *   A response object containing information about the request.
   */
  function queryPinApi($endpoint, $variables) {

    // The endpoint URL we will query.
    $endpoint_url = $this->getApiBase() . '/' . $endpoint . '/';

    // Add some standard variables to our request variables.
    $request_variables = array(
      'ip_address' => $_SERVER['REMOTE_ADDR'],
    ) + $variables;

    // Post to our endpoint with all our variables.
    $response = $this->jsonPostRequest($endpoint_url, $request_variables);

    // Return a newly constructed response object.
    return new CommercePinResponse($response);

  }


  /**
   * Make a JSON post request.
   *
   * @param $url
   *   The URL to POST to.
   * @param $data
   *   An array of data to be converted and posted as JSON.
   * @return Array
   *   An array representing the response from the server.
   */
  function jsonPostRequest($url, $data) {
    $encoded_request_variables = json_encode($data);
    $http_response = drupal_http_request(
      $url,
      array(
        'method' => 'POST',
        'headers' => array(
          'Content-Length' => strlen($encoded_request_variables),
          'Content-Type' => 'application/json',
        ),
        'data' => $encoded_request_variables,
      )
    );

    return json_decode($http_response->data);
  }

}
